package purang

import (
	"crypto/md5"
	"crypto/sha512"
	"encoding/hex"
	"fmt"
	"sort"
	"strings"
	"time"
)

// Request 请求参数结构体
type Request struct {
	CompanyCode string    `json:"companyCode"` // 公司代码
	StartTime   time.Time `json:"startTime"`   // 开始时间
	EndTime     time.Time `json:"endTime"`     // 结束时间
}

// Response 响应结构体
type Response struct {
	Code    int         `json:"code"`    // 响应代码
	Message string      `json:"message"` // 响应消息
	Data    []DataItem  `json:"data"`    // 数据列表
}

// DataItem 数据项结构体
type DataItem struct {
	DeviceID    string    `json:"deviceId"`    // 设备ID
	DeviceName  string    `json:"deviceName"`  // 设备名称
	DataTime    time.Time `json:"dataTime"`    // 数据时间
	Temperature float64   `json:"temperature"` // 温度值
	Humidity    float64   `json:"humidity"`    // 湿度值
	Status      int       `json:"status"`      // 设备状态
}

// DirectPasteRequest 直贴报价/成交行情请求参数
type DirectPasteRequest struct {
	StartDate string `json:"startDate"` // 开始日期，格式：yyyy-MM-dd
	EndDate   string `json:"endDate"`   // 结束日期，格式：yyyy-MM-dd
	PageNo    int    `json:"pageNo"`    // 页码，从1开始
	PageSize  int    `json:"pageSize"`  // 每页记录数
}

// DirectPasteResponse 直贴报价/成交行情响应
type DirectPasteResponse struct {
	Code      int                `json:"code"`      // 响应代码
	Message   string             `json:"message"`   // 响应消息
	Data      []DirectPasteData  `json:"data"`      // 数据列表
	Total     int               `json:"total"`     // 总记录数
	PageNo    int               `json:"pageNo"`    // 当前页码
	PageSize  int               `json:"pageSize"`  // 每页记录数
}

// DirectPasteData 直贴报价/成交行情数据项
type DirectPasteData struct {
	TradeDate     string  `json:"tradeDate"`     // 交易日期，格式：yyyy-MM-dd
	BankName      string  `json:"bankName"`      // 银行名称
	MinRate       float64 `json:"minRate"`       // 最低利率（%）
	MaxRate       float64 `json:"maxRate"`       // 最高利率（%）
	AverageRate   float64 `json:"averageRate"`   // 平均利率（%）
	TotalAmount   float64 `json:"totalAmount"`   // 总金额（万元）
	DealCount     int     `json:"dealCount"`     // 成交笔数
	BusinessType  string  `json:"businessType"`  // 业务类型（报价/成交）
	Deadline      int     `json:"deadline"`      // 期限（天）
}

// ErrorResponse 错误响应结构体
type ErrorResponse struct {
	Code    int    `json:"code"`
	Message string `json:"message"`
}

// BaseRequest 基础请求结构
type BaseRequest struct {
	SpNo     string `json:"sp_no"`     // 用户编号（6位数字）
	SignType string `json:"sign_type"` // 签名类型：1（MD5），2（SHA512）
	ReqID    string `json:"req_id"`    // 平台流水号
	Sign     string `json:"sign"`      // 签名
}

// BaseResponse 基础响应结构
type BaseResponse struct {
	Code     string      `json:"code"`      // 响应状态码
	Message  string      `json:"message"`   // 响应状态描述
	SignType string      `json:"sign_type"` // 签名类型
	ReqID    string      `json:"req_id"`    // 平台流水号
	Data     interface{} `json:"data"`      // 数据内容
}

// SignConfig 签名配置
type SignConfig struct {
	Key      string // API密钥
	SignType string // 签名类型：1（MD5），2（SHA512）
}

// GenerateSign 生成签名
func GenerateSign(params map[string]string, config SignConfig) (string, error) {
	// 1. 按参数名称字符升序排列
	var keys []string
	for k := range params {
		if k != "sign" { // 排除sign参数
			keys = append(keys, k)
		}
	}
	sort.Strings(keys)

	// 2. 拼接待签名字符串
	var signParts []string
	for _, k := range keys {
		if params[k] != "" { // 空值不参与签名
			signParts = append(signParts, fmt.Sprintf("%s=%s", k, params[k]))
		}
	}
	
	// 3. 添加密钥
	signParts = append(signParts, fmt.Sprintf("key=%s", config.Key))
	signStr := strings.Join(signParts, "&")
	//fmt.Println("生成明文",signStr)
	// 4. 根据签名类型选择相应的摘要算法
	var signBytes []byte
	switch config.SignType {
	case "1": // MD5
		hash := md5.New()
		hash.Write([]byte(signStr))
		signBytes = hash.Sum(nil)
	case "2": // SHA512
		hash := sha512.New()
		hash.Write([]byte(signStr))
		signBytes = hash.Sum(nil)
	default:
		return "", fmt.Errorf("unsupported sign type: %s", config.SignType)
	}

	// 5. 转换为小写十六进制字符串
	return hex.EncodeToString(signBytes), nil
}

// VerifySign 验证签名
func VerifySign(params map[string]string, config SignConfig) (bool, error) {
	originalSign := params["sign"]
	if originalSign == "" {
		return false, fmt.Errorf("sign parameter is missing")
	}

	calculatedSign, err := GenerateSign(params, config)
	if err != nil {
		return false, err
	}

	return strings.EqualFold(originalSign, calculatedSign), nil
} 