package purang

import (
	"bytes"
	"encoding/json"
	"eta/eta_crawler/utils"
	"fmt"
	"io"

	"net/http"
	"net/url"
	"strings"
	"time"
)

// Client 普兰数据平台客户端
type Client struct {
	BaseURL    string     // API基础URL
	SpNo       string     // 用户编号
	SignConfig SignConfig // 签名配置
	HTTPClient *http.Client
}

// NewClient 创建新的客户端实例
func NewClient(baseURL, spNo, apiKey, signType string) *Client {
	return &Client{
		BaseURL: baseURL,
		SpNo:    spNo,
		SignConfig: SignConfig{
			Key:      apiKey,
			SignType: signType,
		},
		HTTPClient: &http.Client{
			Timeout: 30 * time.Second,
		},
	}
}

// GenerateReqID 生成请求流水号
func (c *Client) GenerateReqID() string {
	// 用户编号（6位）+ 交易日期（8位yyyyMMdd）+ 流水序号（19位）
	date := time.Now().Format("20060102")
	sequence := fmt.Sprintf("%019d", time.Now().UnixNano())
	return fmt.Sprintf("%s%s%s", c.SpNo, date, sequence)
}

// DoRequest 发送请求到普兰数据平台
func (c *Client) DoRequest(path string, params map[string]string) (*BaseResponse, error) {
	// 1. 添加基础参数
	if params == nil {
		params = make(map[string]string)
	}
	params["sp_no"] = c.SpNo
	params["sign_type"] = c.SignConfig.SignType
	params["req_id"] = c.GenerateReqID()

	// 2. 生成签名
	sign, err := GenerateSign(params, c.SignConfig)
	if err != nil {
		return nil, fmt.Errorf("generate sign failed: %v", err)
	}
	params["sign"] = sign

	// 3. 构建请求表单
	form := url.Values{}
	for k, v := range params {
		form.Add(k, v)
	}

	// 4. 发送请求
	reqURL := fmt.Sprintf("%s%s", c.BaseURL, path)
	req, err := http.NewRequest("POST", reqURL, strings.NewReader(form.Encode()))
	if err != nil {
		return nil, fmt.Errorf("create request failed: %v", err)
	}

	req.Header.Set("Content-Type", "application/x-www-form-urlencoded")

	// 打印请求参数
	//fmt.Println("请求参数",params)
	resp, err := c.HTTPClient.Do(req)
	if err != nil {
		return nil, fmt.Errorf("do request failed: %v", err)
	}
	defer resp.Body.Close()

	// 5. 读取响应
	body, err := io.ReadAll(resp.Body)
	if err != nil {
		return nil, fmt.Errorf("read response body failed: %v", err)
	}
	
	// bodystr := `{"req_id":"100021202503051741155366448842000","code":"10000","data":[{"bbabCountry12m":null,"date":"20241202","ebaCountry6m":"0.88","bbaUrban6m":"0.81","ebaRural12m":null,"bbaRural3m":"0.55","ebaRural3m":"1.0","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"0.3","ebaUrban6m":"0.99","bbabRural3m":"0.7","bbabUrban3m":"0.41","ebaCountry12m":null,"bbaCountry6m":"0.7","ebaUrban3m":"0.81","ebaCountry3m":"0.7","bbaUrban3m":"0.36","bbaCountry12m":null,"bbaRural6m":"1.0","bbabCountry6m":"0.75","bbaRural12m":null,"bbabRural6m":"1.15","ebaRural6m":"1.2","bbabUrban12m":null,"bbaCountry3m":"0.25","bbaUrban12m":null,"bbabUrban6m":"0.86"},{"bbabCountry12m":null,"date":"20241203","ebaCountry6m":"0.85","bbaUrban6m":"0.8","ebaRural12m":null,"bbaRural3m":"0.55","ebaRural3m":"1.0","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"0.28","ebaUrban6m":"0.96","bbabRural3m":"0.7","bbabUrban3m":"0.49","ebaCountry12m":null,"bbaCountry6m":"0.69","ebaUrban3m":"0.81","ebaCountry3m":"0.7","bbaUrban3m":"0.44","bbaCountry12m":null,"bbaRural6m":"1.0","bbabCountry6m":"0.74","bbaRural12m":null,"bbabRural6m":"1.15","ebaRural6m":"1.15","bbabUrban12m":null,"bbaCountry3m":"0.23","bbaUrban12m":null,"bbabUrban6m":"0.85"},{"bbabCountry12m":null,"date":"20241204","ebaCountry6m":"0.85","bbaUrban6m":"0.78","ebaRural12m":null,"bbaRural3m":"0.46","ebaRural3m":"1.0","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"0.21","ebaUrban6m":"0.96","bbabRural3m":"0.61","bbabUrban3m":"0.32","ebaCountry12m":null,"bbaCountry6m":"0.67","ebaUrban3m":"0.81","ebaCountry3m":"0.7","bbaUrban3m":"0.27","bbaCountry12m":null,"bbaRural6m":"0.97","bbabCountry6m":"0.72","bbaRural12m":null,"bbabRural6m":"1.12","ebaRural6m":"1.15","bbabUrban12m":null,"bbaCountry3m":"0.16","bbaUrban12m":null,"bbabUrban6m":"0.83"},{"bbabCountry12m":null,"date":"20241205","ebaCountry6m":"0.85","bbaUrban6m":"0.79","ebaRural12m":null,"bbaRural3m":"0.45","ebaRural3m":"1.0","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"0.2","ebaUrban6m":"0.96","bbabRural3m":"0.6","bbabUrban3m":"0.31","ebaCountry12m":null,"bbaCountry6m":"0.68","ebaUrban3m":"0.81","ebaCountry3m":"0.7","bbaUrban3m":"0.26","bbaCountry12m":null,"bbaRural6m":"0.98","bbabCountry6m":"0.73","bbaRural12m":null,"bbabRural6m":"1.13","ebaRural6m":"1.15","bbabUrban12m":null,"bbaCountry3m":"0.15","bbaUrban12m":null,"bbabUrban6m":"0.84"},{"bbabCountry12m":null,"date":"20241206","ebaCountry6m":"0.88","bbaUrban6m":"0.82","ebaRural12m":null,"bbaRural3m":"0.47","ebaRural3m":"1.05","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"0.22","ebaUrban6m":"0.99","bbabRural3m":"0.62","bbabUrban3m":"0.33","ebaCountry12m":null,"bbaCountry6m":"0.71","ebaUrban3m":"0.83","ebaCountry3m":"0.72","bbaUrban3m":"0.28","bbaCountry12m":null,"bbaRural6m":"1.01","bbabCountry6m":"0.76","bbaRural12m":null,"bbabRural6m":"1.16","ebaRural6m":"1.18","bbabUrban12m":null,"bbaCountry3m":"0.17","bbaUrban12m":null,"bbabUrban6m":"0.87"},{"bbabCountry12m":null,"date":"20241209","ebaCountry6m":"0.9","bbaUrban6m":"0.86","ebaRural12m":null,"bbaRural3m":"0.47","ebaRural3m":"1.05","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"0.22","ebaUrban6m":"1.01","bbabRural3m":"0.62","bbabUrban3m":"0.33","ebaCountry12m":null,"bbaCountry6m":"0.75","ebaUrban3m":"0.83","ebaCountry3m":"0.72","bbaUrban3m":"0.28","bbaCountry12m":null,"bbaRural6m":"1.05","bbabCountry6m":"0.8","bbaRural12m":null,"bbabRural6m":"1.2","ebaRural6m":"1.2","bbabUrban12m":null,"bbaCountry3m":"0.17","bbaUrban12m":null,"bbabUrban6m":"0.91"},{"bbabCountry12m":null,"date":"20241210","ebaCountry6m":"0.91","bbaUrban6m":"0.9","ebaRural12m":null,"bbaRural3m":"0.5","ebaRural3m":"1.05","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"0.24","ebaUrban6m":"1.02","bbabRural3m":"0.65","bbabUrban3m":"0.35","ebaCountry12m":null,"bbaCountry6m":"0.79","ebaUrban3m":"0.85","ebaCountry3m":"0.74","bbaUrban3m":"0.3","bbaCountry12m":null,"bbaRural6m":"1.09","bbabCountry6m":"0.84","bbaRural12m":null,"bbabRural6m":"1.24","ebaRural6m":"1.21","bbabUrban12m":null,"bbaCountry3m":"0.19","bbaUrban12m":null,"bbabUrban6m":"0.95"},{"bbabCountry12m":null,"date":"20241211","ebaCountry6m":"0.91","bbaUrban6m":"0.91","ebaRural12m":null,"bbaRural3m":"0.5","ebaRural3m":"1.05","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"0.24","ebaUrban6m":"1.02","bbabRural3m":"0.65","bbabUrban3m":"0.35","ebaCountry12m":null,"bbaCountry6m":"0.8","ebaUrban3m":"0.86","ebaCountry3m":"0.75","bbaUrban3m":"0.3","bbaCountry12m":null,"bbaRural6m":"1.1","bbabCountry6m":"0.85","bbaRural12m":null,"bbabRural6m":"1.25","ebaRural6m":"1.21","bbabUrban12m":null,"bbaCountry3m":"0.19","bbaUrban12m":null,"bbabUrban6m":"0.96"},{"bbabCountry12m":null,"date":"20241212","ebaCountry6m":"0.9","bbaUrban6m":"0.87","ebaRural12m":null,"bbaRural3m":"0.5","ebaRural3m":"1.05","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"0.24","ebaUrban6m":"1.01","bbabRural3m":"0.65","bbabUrban3m":"0.35","ebaCountry12m":null,"bbaCountry6m":"0.76","ebaUrban3m":"0.86","ebaCountry3m":"0.75","bbaUrban3m":"0.3","bbaCountry12m":null,"bbaRural6m":"1.06","bbabCountry6m":"0.81","bbaRural12m":null,"bbabRural6m":"1.21","ebaRural6m":"1.2","bbabUrban12m":null,"bbaCountry3m":"0.19","bbaUrban12m":null,"bbabUrban6m":"0.92"},{"bbabCountry12m":null,"date":"20241213","ebaCountry6m":"0.85","bbaUrban6m":"0.84","ebaRural12m":null,"bbaRural3m":"0.5","ebaRural3m":"1.05","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"0.22","ebaUrban6m":"0.96","bbabRural3m":"0.65","bbabUrban3m":"0.33","ebaCountry12m":null,"bbaCountry6m":"0.73","ebaUrban3m":"0.85","ebaCountry3m":"0.74","bbaUrban3m":"0.28","bbaCountry12m":null,"bbaRural6m":"1.03","bbabCountry6m":"0.78","bbaRural12m":null,"bbabRural6m":"1.18","ebaRural6m":"1.15","bbabUrban12m":null,"bbaCountry3m":"0.17","bbaUrban12m":null,"bbabUrban6m":"0.89"},{"bbabCountry12m":null,"date":"20241216","ebaCountry6m":"0.9","bbaUrban6m":"0.85","ebaRural12m":null,"bbaRural3m":"0.45","ebaRural3m":"1.05","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"0.2","ebaUrban6m":"1.01","bbabRural3m":"0.6","bbabUrban3m":"0.31","ebaCountry12m":null,"bbaCountry6m":"0.74","ebaUrban3m":"0.83","ebaCountry3m":"0.72","bbaUrban3m":"0.26","bbaCountry12m":null,"bbaRural6m":"1.04","bbabCountry6m":"0.79","bbaRural12m":null,"bbabRural6m":"1.19","ebaRural6m":"1.2","bbabUrban12m":null,"bbaCountry3m":"0.15","bbaUrban12m":null,"bbabUrban6m":"0.9"},{"bbabCountry12m":null,"date":"20241217","ebaCountry6m":"0.88","bbaUrban6m":"0.85","ebaRural12m":null,"bbaRural3m":"0.45","ebaRural3m":"1.0","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"0.17","ebaUrban6m":"0.99","bbabRural3m":"0.6","bbabUrban3m":"0.28","ebaCountry12m":null,"bbaCountry6m":"0.74","ebaUrban3m":"0.81","ebaCountry3m":"0.7","bbaUrban3m":"0.23","bbaCountry12m":null,"bbaRural6m":"1.04","bbabCountry6m":"0.79","bbaRural12m":null,"bbabRural6m":"1.19","ebaRural6m":"1.2","bbabUrban12m":null,"bbaCountry3m":"0.12","bbaUrban12m":null,"bbabUrban6m":"0.9"},{"bbabCountry12m":null,"date":"20241218","ebaCountry6m":"0.91","bbaUrban6m":"0.84","ebaRural12m":null,"bbaRural3m":"0.4","ebaRural3m":"1.0","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"0.13","ebaUrban6m":"1.01","bbabRural3m":"0.55","bbabUrban3m":"0.24","ebaCountry12m":null,"bbaCountry6m":"0.73","ebaUrban3m":"0.78","ebaCountry3m":"0.67","bbaUrban3m":"0.19","bbaCountry12m":null,"bbaRural6m":"1.03","bbabCountry6m":"0.78","bbaRural12m":null,"bbabRural6m":"1.18","ebaRural6m":"1.2","bbabUrban12m":null,"bbaCountry3m":"0.08","bbaUrban12m":null,"bbabUrban6m":"0.89"},{"bbabCountry12m":null,"date":"20241219","ebaCountry6m":"0.9","bbaUrban6m":"0.81","ebaRural12m":null,"bbaRural3m":"0.35","ebaRural3m":"0.95","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"0.09","ebaUrban6m":"1.01","bbabRural3m":"0.5","bbabUrban3m":"0.2","ebaCountry12m":null,"bbaCountry6m":"0.7","ebaUrban3m":"0.76","ebaCountry3m":"0.65","bbaUrban3m":"0.15","bbaCountry12m":null,"bbaRural6m":"1.0","bbabCountry6m":"0.75","bbaRural12m":null,"bbabRural6m":"1.15","ebaRural6m":"1.2","bbabUrban12m":null,"bbaCountry3m":"0.04","bbaUrban12m":null,"bbabUrban6m":"0.86"},{"bbabCountry12m":null,"date":"20241220","ebaCountry6m":"0.9","bbaUrban6m":"0.81","ebaRural12m":null,"bbaRural3m":"0.35","ebaRural3m":"0.95","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"0.09","ebaUrban6m":"1.01","bbabRural3m":"0.5","bbabUrban3m":"0.2","ebaCountry12m":null,"bbaCountry6m":"0.7","ebaUrban3m":"0.76","ebaCountry3m":"0.65","bbaUrban3m":"0.15","bbaCountry12m":null,"bbaRural6m":"1.0","bbabCountry6m":"0.75","bbaRural12m":null,"bbabRural6m":"1.15","ebaRural6m":"1.2","bbabUrban12m":null,"bbaCountry3m":"0.04","bbaUrban12m":null,"bbabUrban6m":"0.86"},{"bbabCountry12m":null,"date":"20241223","ebaCountry6m":"0.88","bbaUrban6m":"0.78","ebaRural12m":null,"bbaRural3m":"0.36","ebaRural3m":"0.95","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"0.1","ebaUrban6m":"0.99","bbabRural3m":"0.51","bbabUrban3m":"0.21","ebaCountry12m":null,"bbaCountry6m":"0.67","ebaUrban3m":"0.76","ebaCountry3m":"0.65","bbaUrban3m":"0.16","bbaCountry12m":null,"bbaRural6m":"0.97","bbabCountry6m":"0.72","bbaRural12m":null,"bbabRural6m":"1.12","ebaRural6m":"1.19","bbabUrban12m":null,"bbaCountry3m":"0.05","bbaUrban12m":null,"bbabUrban6m":"0.83"},{"bbabCountry12m":null,"date":"20241224","ebaCountry6m":"0.85","bbaUrban6m":"0.75","ebaRural12m":null,"bbaRural3m":"0.35","ebaRural3m":"0.95","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"0.08","ebaUrban6m":"0.96","bbabRural3m":"0.5","bbabUrban3m":"0.19","ebaCountry12m":null,"bbaCountry6m":"0.64","ebaUrban3m":"0.73","ebaCountry3m":"0.62","bbaUrban3m":"0.14","bbaCountry12m":null,"bbaRural6m":"0.95","bbabCountry6m":"0.69","bbaRural12m":null,"bbabRural6m":"1.1","ebaRural6m":"1.15","bbabUrban12m":null,"bbaCountry3m":"0.03","bbaUrban12m":null,"bbabUrban6m":"0.8"},{"bbabCountry12m":null,"date":"20241225","ebaCountry6m":"0.83","bbaUrban6m":"0.69","ebaRural12m":null,"bbaRural3m":"0.31","ebaRural3m":"0.9","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"0.06","ebaUrban6m":"0.94","bbabRural3m":"0.46","bbabUrban3m":"0.17","ebaCountry12m":null,"bbaCountry6m":"0.58","ebaUrban3m":"0.71","ebaCountry3m":"0.6","bbaUrban3m":"0.12","bbaCountry12m":null,"bbaRural6m":"0.88","bbabCountry6m":"0.63","bbaRural12m":null,"bbabRural6m":"1.03","ebaRural6m":"1.15","bbabUrban12m":null,"bbaCountry3m":"0.01","bbaUrban12m":null,"bbabUrban6m":"0.74"},{"bbabCountry12m":null,"date":"20241226","ebaCountry6m":"0.92","bbaUrban6m":"0.91","ebaRural12m":null,"bbaRural3m":"0.36","ebaRural3m":"0.95","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"0.11","ebaUrban6m":"1.03","bbabRural3m":"0.51","bbabUrban3m":"0.22","ebaCountry12m":null,"bbaCountry6m":"0.8","ebaUrban3m":"0.76","ebaCountry3m":"0.65","bbaUrban3m":"0.17","bbaCountry12m":null,"bbaRural6m":"1.1","bbabCountry6m":"0.85","bbaRural12m":null,"bbabRural6m":"1.25","ebaRural6m":"1.22","bbabUrban12m":null,"bbaCountry3m":"0.06","bbaUrban12m":null,"bbabUrban6m":"0.96"},{"bbabCountry12m":null,"date":"20241227","ebaCountry6m":"1.33","bbaUrban6m":"1.36","ebaRural12m":null,"bbaRural3m":"1.4","ebaRural3m":"1.45","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.12","ebaUrban6m":"1.44","bbabRural3m":"1.55","bbabUrban3m":"1.23","ebaCountry12m":null,"bbaCountry6m":"1.25","ebaUrban3m":"1.26","ebaCountry3m":"1.15","bbaUrban3m":"1.18","bbaCountry12m":null,"bbaRural6m":"1.55","bbabCountry6m":"1.3","bbaRural12m":null,"bbabRural6m":"1.7","ebaRural6m":"1.65","bbabUrban12m":null,"bbaCountry3m":"1.07","bbaUrban12m":null,"bbabUrban6m":"1.41"},{"bbabCountry12m":null,"date":"20241230","ebaCountry6m":"1.7","bbaUrban6m":"1.7","ebaRural12m":null,"bbaRural3m":"2.0","ebaRural3m":"2.1","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.72","ebaUrban6m":"1.81","bbabRural3m":"2.15","bbabUrban3m":"1.83","ebaCountry12m":null,"bbaCountry6m":"1.59","ebaUrban3m":"1.89","ebaCountry3m":"1.78","bbaUrban3m":"1.78","bbaCountry12m":null,"bbaRural6m":"1.89","bbabCountry6m":"1.64","bbaRural12m":null,"bbabRural6m":"2.04","ebaRural6m":"2.0","bbabUrban12m":null,"bbaCountry3m":"1.67","bbaUrban12m":null,"bbabUrban6m":"1.75"},{"bbabCountry12m":null,"date":"20241231","ebaCountry6m":"1.7","bbaUrban6m":"1.41","ebaRural12m":null,"bbaRural3m":"1.3","ebaRural3m":"2.1","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.05","ebaUrban6m":"1.81","bbabRural3m":"1.45","bbabUrban3m":"1.16","ebaCountry12m":null,"bbaCountry6m":"1.3","ebaUrban3m":"1.89","ebaCountry3m":"1.78","bbaUrban3m":"1.11","bbaCountry12m":null,"bbaRural6m":"1.6","bbabCountry6m":"1.35","bbaRural12m":null,"bbabRural6m":"1.75","ebaRural6m":"2.0","bbabUrban12m":null,"bbaCountry3m":"1.0","bbaUrban12m":null,"bbabUrban6m":"1.46"},{"bbabCountry12m":null,"date":"20250102","ebaCountry6m":"1.75","bbaUrban6m":"1.71","ebaRural12m":null,"bbaRural3m":"2.0","ebaRural3m":"2.15","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.73","ebaUrban6m":"1.86","bbabRural3m":"2.15","bbabUrban3m":"1.84","ebaCountry12m":null,"bbaCountry6m":"1.6","ebaUrban3m":"1.96","ebaCountry3m":"1.85","bbaUrban3m":"1.79","bbaCountry12m":null,"bbaRural6m":"1.9","bbabCountry6m":"1.65","bbaRural12m":null,"bbabRural6m":"2.05","ebaRural6m":"2.05","bbabUrban12m":null,"bbaCountry3m":"1.68","bbaUrban12m":null,"bbabUrban6m":"1.76"},{"bbabCountry12m":null,"date":"20250103","ebaCountry6m":"1.65","bbaUrban6m":"1.59","ebaRural12m":null,"bbaRural3m":"1.95","ebaRural3m":"2.15","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.7","ebaUrban6m":"1.76","bbabRural3m":"2.1","bbabUrban3m":"1.81","ebaCountry12m":null,"bbaCountry6m":"1.48","ebaUrban3m":"1.94","ebaCountry3m":"1.83","bbaUrban3m":"1.76","bbaCountry12m":null,"bbaRural6m":"1.78","bbabCountry6m":"1.53","bbaRural12m":null,"bbabRural6m":"1.93","ebaRural6m":"1.95","bbabUrban12m":null,"bbaCountry3m":"1.65","bbaUrban12m":null,"bbabUrban6m":"1.64"},{"bbabCountry12m":null,"date":"20250106","ebaCountry6m":"1.57","bbaUrban6m":"1.5","ebaRural12m":null,"bbaRural3m":"1.9","ebaRural3m":"2.06","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.65","ebaUrban6m":"1.68","bbabRural3m":"2.05","bbabUrban3m":"1.76","ebaCountry12m":null,"bbaCountry6m":"1.39","ebaUrban3m":"1.87","ebaCountry3m":"1.76","bbaUrban3m":"1.71","bbaCountry12m":null,"bbaRural6m":"1.7","bbabCountry6m":"1.44","bbaRural12m":null,"bbabRural6m":"1.85","ebaRural6m":"1.87","bbabUrban12m":null,"bbaCountry3m":"1.6","bbaUrban12m":null,"bbabUrban6m":"1.55"},{"bbabCountry12m":null,"date":"20250107","ebaCountry6m":"1.58","bbaUrban6m":"1.56","ebaRural12m":null,"bbaRural3m":"1.9","ebaRural3m":"2.06","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.65","ebaUrban6m":"1.69","bbabRural3m":"2.05","bbabUrban3m":"1.76","ebaCountry12m":null,"bbaCountry6m":"1.45","ebaUrban3m":"1.87","ebaCountry3m":"1.76","bbaUrban3m":"1.71","bbaCountry12m":null,"bbaRural6m":"1.75","bbabCountry6m":"1.5","bbaRural12m":null,"bbabRural6m":"1.9","ebaRural6m":"1.88","bbabUrban12m":null,"bbaCountry3m":"1.6","bbaUrban12m":null,"bbabUrban6m":"1.61"},{"bbabCountry12m":null,"date":"20250108","ebaCountry6m":"1.52","bbaUrban6m":"1.51","ebaRural12m":null,"bbaRural3m":"1.91","ebaRural3m":"2.12","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.66","ebaUrban6m":"1.63","bbabRural3m":"2.06","bbabUrban3m":"1.77","ebaCountry12m":null,"bbaCountry6m":"1.4","ebaUrban3m":"1.93","ebaCountry3m":"1.82","bbaUrban3m":"1.72","bbaCountry12m":null,"bbaRural6m":"1.7","bbabCountry6m":"1.45","bbaRural12m":null,"bbabRural6m":"1.85","ebaRural6m":"1.82","bbabUrban12m":null,"bbaCountry3m":"1.61","bbaUrban12m":null,"bbabUrban6m":"1.56"},{"bbabCountry12m":null,"date":"20250109","ebaCountry6m":"1.53","bbaUrban6m":"1.53","ebaRural12m":null,"bbaRural3m":"1.92","ebaRural3m":"2.13","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.67","ebaUrban6m":"1.64","bbabRural3m":"2.07","bbabUrban3m":"1.78","ebaCountry12m":null,"bbaCountry6m":"1.42","ebaUrban3m":"1.94","ebaCountry3m":"1.83","bbaUrban3m":"1.73","bbaCountry12m":null,"bbaRural6m":"1.72","bbabCountry6m":"1.47","bbaRural12m":null,"bbabRural6m":"1.87","ebaRural6m":"1.83","bbabUrban12m":null,"bbaCountry3m":"1.62","bbaUrban12m":null,"bbabUrban6m":"1.58"},{"bbabCountry12m":null,"date":"20250110","ebaCountry6m":"1.53","bbaUrban6m":"1.52","ebaRural12m":null,"bbaRural3m":"1.95","ebaRural3m":"2.13","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.68","ebaUrban6m":"1.64","bbabRural3m":"2.1","bbabUrban3m":"1.79","ebaCountry12m":null,"bbaCountry6m":"1.41","ebaUrban3m":"1.94","ebaCountry3m":"1.83","bbaUrban3m":"1.74","bbaCountry12m":null,"bbaRural6m":"1.71","bbabCountry6m":"1.46","bbaRural12m":null,"bbabRural6m":"1.86","ebaRural6m":"1.83","bbabUrban12m":null,"bbaCountry3m":"1.63","bbaUrban12m":null,"bbabUrban6m":"1.57"},{"bbabCountry12m":null,"date":"20250113","ebaCountry6m":"1.47","bbaUrban6m":"1.5","ebaRural12m":null,"bbaRural3m":"1.95","ebaRural3m":"2.12","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.68","ebaUrban6m":"1.58","bbabRural3m":"2.1","bbabUrban3m":"1.79","ebaCountry12m":null,"bbaCountry6m":"1.39","ebaUrban3m":"1.93","ebaCountry3m":"1.82","bbaUrban3m":"1.74","bbaCountry12m":null,"bbaRural6m":"1.7","bbabCountry6m":"1.44","bbaRural12m":null,"bbabRural6m":"1.85","ebaRural6m":"1.77","bbabUrban12m":null,"bbaCountry3m":"1.63","bbaUrban12m":null,"bbabUrban6m":"1.55"},{"bbabCountry12m":null,"date":"20250114","ebaCountry6m":"1.45","bbaUrban6m":"1.48","ebaRural12m":null,"bbaRural3m":"1.95","ebaRural3m":"2.12","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.68","ebaUrban6m":"1.56","bbabRural3m":"2.1","bbabUrban3m":"1.79","ebaCountry12m":null,"bbaCountry6m":"1.37","ebaUrban3m":"1.93","ebaCountry3m":"1.82","bbaUrban3m":"1.74","bbaCountry12m":null,"bbaRural6m":"1.67","bbabCountry6m":"1.42","bbaRural12m":null,"bbabRural6m":"1.82","ebaRural6m":"1.75","bbabUrban12m":null,"bbaCountry3m":"1.63","bbaUrban12m":null,"bbabUrban6m":"1.53"},{"bbabCountry12m":null,"date":"20250115","ebaCountry6m":"1.45","bbaUrban6m":"1.48","ebaRural12m":null,"bbaRural3m":"1.95","ebaRural3m":"2.12","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.69","ebaUrban6m":"1.56","bbabRural3m":"2.1","bbabUrban3m":"1.8","ebaCountry12m":null,"bbaCountry6m":"1.37","ebaUrban3m":"1.93","ebaCountry3m":"1.82","bbaUrban3m":"1.75","bbaCountry12m":null,"bbaRural6m":"1.67","bbabCountry6m":"1.42","bbaRural12m":null,"bbabRural6m":"1.82","ebaRural6m":"1.75","bbabUrban12m":null,"bbaCountry3m":"1.64","bbaUrban12m":null,"bbabUrban6m":"1.53"},{"bbabCountry12m":null,"date":"20250116","ebaCountry6m":"1.49","bbaUrban6m":"1.52","ebaRural12m":null,"bbaRural3m":"2.0","ebaRural3m":"2.15","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.72","ebaUrban6m":"1.6","bbabRural3m":"2.15","bbabUrban3m":"1.83","ebaCountry12m":null,"bbaCountry6m":"1.41","ebaUrban3m":"1.96","ebaCountry3m":"1.85","bbaUrban3m":"1.78","bbaCountry12m":null,"bbaRural6m":"1.71","bbabCountry6m":"1.46","bbaRural12m":null,"bbabRural6m":"1.86","ebaRural6m":"1.8","bbabUrban12m":null,"bbaCountry3m":"1.67","bbaUrban12m":null,"bbabUrban6m":"1.57"},{"bbabCountry12m":null,"date":"20250117","ebaCountry6m":"1.47","bbaUrban6m":"1.49","ebaRural12m":null,"bbaRural3m":"2.0","ebaRural3m":"2.16","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.75","ebaUrban6m":"1.58","bbabRural3m":"2.15","bbabUrban3m":"1.86","ebaCountry12m":null,"bbaCountry6m":"1.38","ebaUrban3m":"1.97","ebaCountry3m":"1.86","bbaUrban3m":"1.81","bbaCountry12m":null,"bbaRural6m":"1.68","bbabCountry6m":"1.43","bbaRural12m":null,"bbabRural6m":"1.83","ebaRural6m":"1.77","bbabUrban12m":null,"bbaCountry3m":"1.7","bbaUrban12m":null,"bbabUrban6m":"1.54"},{"bbabCountry12m":null,"date":"20250120","ebaCountry6m":"1.42","bbaUrban6m":"1.42","ebaRural12m":null,"bbaRural3m":"1.9","ebaRural3m":"2.05","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.63","ebaUrban6m":"1.53","bbabRural3m":"2.05","bbabUrban3m":"1.74","ebaCountry12m":null,"bbaCountry6m":"1.31","ebaUrban3m":"1.86","ebaCountry3m":"1.75","bbaUrban3m":"1.69","bbaCountry12m":null,"bbaRural6m":"1.61","bbabCountry6m":"1.36","bbaRural12m":null,"bbabRural6m":"1.76","ebaRural6m":"1.75","bbabUrban12m":null,"bbaCountry3m":"1.58","bbaUrban12m":null,"bbabUrban6m":"1.47"},{"bbabCountry12m":null,"date":"20250121","ebaCountry6m":"1.45","bbaUrban6m":"1.48","ebaRural12m":null,"bbaRural3m":"1.95","ebaRural3m":"2.15","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.7","ebaUrban6m":"1.56","bbabRural3m":"2.1","bbabUrban3m":"1.81","ebaCountry12m":null,"bbaCountry6m":"1.37","ebaUrban3m":"1.96","ebaCountry3m":"1.85","bbaUrban3m":"1.76","bbaCountry12m":null,"bbaRural6m":"1.67","bbabCountry6m":"1.42","bbaRural12m":null,"bbabRural6m":"1.82","ebaRural6m":"1.75","bbabUrban12m":null,"bbaCountry3m":"1.65","bbaUrban12m":null,"bbabUrban6m":"1.53"},{"bbabCountry12m":null,"date":"20250122","ebaCountry6m":"1.52","bbaUrban6m":"1.55","ebaRural12m":null,"bbaRural3m":"2.1","ebaRural3m":"2.25","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.85","ebaUrban6m":"1.63","bbabRural3m":"2.25","bbabUrban3m":"1.96","ebaCountry12m":null,"bbaCountry6m":"1.44","ebaUrban3m":"2.06","ebaCountry3m":"1.95","bbaUrban3m":"1.91","bbaCountry12m":null,"bbaRural6m":"1.75","bbabCountry6m":"1.49","bbaRural12m":null,"bbabRural6m":"1.9","ebaRural6m":"1.82","bbabUrban12m":null,"bbaCountry3m":"1.8","bbaUrban12m":null,"bbabUrban6m":"1.6"},{"bbabCountry12m":null,"date":"20250123","ebaCountry6m":"1.52","bbaUrban6m":"1.53","ebaRural12m":null,"bbaRural3m":"2.1","ebaRural3m":"2.25","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.85","ebaUrban6m":"1.63","bbabRural3m":"2.25","bbabUrban3m":"1.96","ebaCountry12m":null,"bbaCountry6m":"1.42","ebaUrban3m":"2.04","ebaCountry3m":"1.93","bbaUrban3m":"1.91","bbaCountry12m":null,"bbaRural6m":"1.72","bbabCountry6m":"1.47","bbaRural12m":null,"bbabRural6m":"1.87","ebaRural6m":"1.82","bbabUrban12m":null,"bbaCountry3m":"1.8","bbaUrban12m":null,"bbabUrban6m":"1.58"},{"bbabCountry12m":null,"date":"20250124","ebaCountry6m":"1.61","bbaUrban6m":"1.59","ebaRural12m":null,"bbaRural3m":"2.25","ebaRural3m":"2.4","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"2.0","ebaUrban6m":"1.72","bbabRural3m":"2.4","bbabUrban3m":"2.11","ebaCountry12m":null,"bbaCountry6m":"1.48","ebaUrban3m":"2.21","ebaCountry3m":"2.1","bbaUrban3m":"2.06","bbaCountry12m":null,"bbaRural6m":"1.78","bbabCountry6m":"1.53","bbaRural12m":null,"bbabRural6m":"1.93","ebaRural6m":"1.91","bbabUrban12m":null,"bbaCountry3m":"1.95","bbaUrban12m":null,"bbabUrban6m":"1.64"},{"bbabCountry12m":null,"date":"20250126","ebaCountry6m":"1.55","bbaUrban6m":"1.48","ebaRural12m":null,"bbaRural3m":"2.2","ebaRural3m":"2.35","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.95","ebaUrban6m":"1.66","bbabRural3m":"2.35","bbabUrban3m":"2.06","ebaCountry12m":null,"bbaCountry6m":"1.37","ebaUrban3m":"2.16","ebaCountry3m":"2.05","bbaUrban3m":"2.01","bbaCountry12m":null,"bbaRural6m":"1.67","bbabCountry6m":"1.42","bbaRural12m":null,"bbabRural6m":"1.82","ebaRural6m":"1.85","bbabUrban12m":null,"bbaCountry3m":"1.9","bbaUrban12m":null,"bbabUrban6m":"1.53"},{"bbabCountry12m":null,"date":"20250127","ebaCountry6m":"1.68","bbaUrban6m":"1.41","ebaRural12m":null,"bbaRural3m":"2.05","ebaRural3m":"2.55","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.8","ebaUrban6m":"1.79","bbabRural3m":"2.2","bbabUrban3m":"1.91","ebaCountry12m":null,"bbaCountry6m":"1.3","ebaUrban3m":"2.36","ebaCountry3m":"2.25","bbaUrban3m":"1.86","bbaCountry12m":null,"bbaRural6m":"1.6","bbabCountry6m":"1.35","bbaRural12m":null,"bbabRural6m":"1.75","ebaRural6m":"1.98","bbabUrban12m":null,"bbaCountry3m":"1.75","bbaUrban12m":null,"bbabUrban6m":"1.46"},{"bbabCountry12m":null,"date":"20250205","ebaCountry6m":"1.19","bbaUrban6m":"1.19","ebaRural12m":null,"bbaRural3m":"1.65","ebaRural3m":"1.75","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.45","ebaUrban6m":"1.3","bbabRural3m":"1.85","bbabUrban3m":"1.56","ebaCountry12m":null,"bbaCountry6m":"1.08","ebaUrban3m":"1.56","ebaCountry3m":"1.45","bbaUrban3m":"1.46","bbaCountry12m":null,"bbaRural6m":"1.38","bbabCountry6m":"1.13","bbaRural12m":null,"bbabRural6m":"1.53","ebaRural6m":"1.5","bbabUrban12m":null,"bbaCountry3m":"1.35","bbaUrban12m":null,"bbabUrban6m":"1.24"},{"bbabCountry12m":null,"date":"20250206","ebaCountry6m":"1.16","bbaUrban6m":"1.16","ebaRural12m":null,"bbaRural3m":"1.6","ebaRural3m":"1.75","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.37","ebaUrban6m":"1.27","bbabRural3m":"1.8","bbabUrban3m":"1.48","ebaCountry12m":null,"bbaCountry6m":"1.05","ebaUrban3m":"1.54","ebaCountry3m":"1.43","bbaUrban3m":"1.38","bbaCountry12m":null,"bbaRural6m":"1.35","bbabCountry6m":"1.1","bbaRural12m":null,"bbabRural6m":"1.5","ebaRural6m":"1.46","bbabUrban12m":null,"bbaCountry3m":"1.27","bbaUrban12m":null,"bbabUrban6m":"1.21"},{"bbabCountry12m":null,"date":"20250207","ebaCountry6m":"1.13","bbaUrban6m":"1.15","ebaRural12m":null,"bbaRural3m":"1.55","ebaRural3m":"1.75","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.35","ebaUrban6m":"1.24","bbabRural3m":"1.75","bbabUrban3m":"1.46","ebaCountry12m":null,"bbaCountry6m":"1.04","ebaUrban3m":"1.53","ebaCountry3m":"1.42","bbaUrban3m":"1.36","bbaCountry12m":null,"bbaRural6m":"1.34","bbabCountry6m":"1.09","bbaRural12m":null,"bbabRural6m":"1.49","ebaRural6m":"1.45","bbabUrban12m":null,"bbaCountry3m":"1.25","bbaUrban12m":null,"bbabUrban6m":"1.2"},{"bbabCountry12m":null,"date":"20250208","ebaCountry6m":"1.18","bbaUrban6m":"1.19","ebaRural12m":null,"bbaRural3m":"1.6","ebaRural3m":"1.76","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.38","ebaUrban6m":"1.29","bbabRural3m":"1.8","bbabUrban3m":"1.49","ebaCountry12m":null,"bbaCountry6m":"1.08","ebaUrban3m":"1.57","ebaCountry3m":"1.46","bbaUrban3m":"1.39","bbaCountry12m":null,"bbaRural6m":"1.38","bbabCountry6m":"1.13","bbaRural12m":null,"bbabRural6m":"1.53","ebaRural6m":"1.48","bbabUrban12m":null,"bbaCountry3m":"1.28","bbaUrban12m":null,"bbabUrban6m":"1.24"},{"bbabCountry12m":null,"date":"20250210","ebaCountry6m":"1.21","bbaUrban6m":"1.24","ebaRural12m":null,"bbaRural3m":"1.7","ebaRural3m":"1.85","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.5","ebaUrban6m":"1.32","bbabRural3m":"1.9","bbabUrban3m":"1.61","ebaCountry12m":null,"bbaCountry6m":"1.13","ebaUrban3m":"1.63","ebaCountry3m":"1.52","bbaUrban3m":"1.51","bbaCountry12m":null,"bbaRural6m":"1.43","bbabCountry6m":"1.18","bbaRural12m":null,"bbabRural6m":"1.58","ebaRural6m":"1.51","bbabUrban12m":null,"bbaCountry3m":"1.4","bbaUrban12m":null,"bbabUrban6m":"1.29"},{"bbabCountry12m":null,"date":"20250211","ebaCountry6m":"1.21","bbaUrban6m":"1.26","ebaRural12m":null,"bbaRural3m":"1.7","ebaRural3m":"1.85","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.5","ebaUrban6m":"1.32","bbabRural3m":"1.9","bbabUrban3m":"1.61","ebaCountry12m":null,"bbaCountry6m":"1.15","ebaUrban3m":"1.63","ebaCountry3m":"1.52","bbaUrban3m":"1.51","bbaCountry12m":null,"bbaRural6m":"1.45","bbabCountry6m":"1.2","bbaRural12m":null,"bbabRural6m":"1.6","ebaRural6m":"1.51","bbabUrban12m":null,"bbaCountry3m":"1.4","bbaUrban12m":null,"bbabUrban6m":"1.31"},{"bbabCountry12m":null,"date":"20250212","ebaCountry6m":"1.25","bbaUrban6m":"1.28","ebaRural12m":null,"bbaRural3m":"1.73","ebaRural3m":"1.85","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.53","ebaUrban6m":"1.36","bbabRural3m":"1.93","bbabUrban3m":"1.64","ebaCountry12m":null,"bbaCountry6m":"1.17","ebaUrban3m":"1.66","ebaCountry3m":"1.55","bbaUrban3m":"1.54","bbaCountry12m":null,"bbaRural6m":"1.47","bbabCountry6m":"1.22","bbaRural12m":null,"bbabRural6m":"1.62","ebaRural6m":"1.55","bbabUrban12m":null,"bbaCountry3m":"1.43","bbaUrban12m":null,"bbabUrban6m":"1.33"},{"bbabCountry12m":null,"date":"20250213","ebaCountry6m":"1.25","bbaUrban6m":"1.28","ebaRural12m":null,"bbaRural3m":"1.73","ebaRural3m":"1.85","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.53","ebaUrban6m":"1.36","bbabRural3m":"1.93","bbabUrban3m":"1.64","ebaCountry12m":null,"bbaCountry6m":"1.17","ebaUrban3m":"1.66","ebaCountry3m":"1.55","bbaUrban3m":"1.54","bbaCountry12m":null,"bbaRural6m":"1.47","bbabCountry6m":"1.22","bbaRural12m":null,"bbabRural6m":"1.62","ebaRural6m":"1.55","bbabUrban12m":null,"bbaCountry3m":"1.43","bbaUrban12m":null,"bbabUrban6m":"1.33"},{"bbabCountry12m":null,"date":"20250214","ebaCountry6m":"1.25","bbaUrban6m":"1.31","ebaRural12m":null,"bbaRural3m":"1.75","ebaRural3m":"1.86","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.55","ebaUrban6m":"1.36","bbabRural3m":"1.95","bbabUrban3m":"1.66","ebaCountry12m":null,"bbaCountry6m":"1.2","ebaUrban3m":"1.67","ebaCountry3m":"1.56","bbaUrban3m":"1.56","bbaCountry12m":null,"bbaRural6m":"1.5","bbabCountry6m":"1.25","bbaRural12m":null,"bbabRural6m":"1.65","ebaRural6m":"1.55","bbabUrban12m":null,"bbaCountry3m":"1.45","bbaUrban12m":null,"bbabUrban6m":"1.36"},{"bbabCountry12m":null,"date":"20250217","ebaCountry6m":"1.26","bbaUrban6m":"1.34","ebaRural12m":null,"bbaRural3m":"1.8","ebaRural3m":"1.9","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.52","ebaUrban6m":"1.37","bbabRural3m":"1.95","bbabUrban3m":"1.63","ebaCountry12m":null,"bbaCountry6m":"1.23","ebaUrban3m":"1.69","ebaCountry3m":"1.58","bbaUrban3m":"1.58","bbaCountry12m":null,"bbaRural6m":"1.53","bbabCountry6m":"1.28","bbaRural12m":null,"bbabRural6m":"1.68","ebaRural6m":"1.56","bbabUrban12m":null,"bbaCountry3m":"1.47","bbaUrban12m":null,"bbabUrban6m":"1.39"},{"bbabCountry12m":null,"date":"20250218","ebaCountry6m":"1.28","bbaUrban6m":"1.35","ebaRural12m":null,"bbaRural3m":"1.82","ebaRural3m":"1.95","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.57","ebaUrban6m":"1.39","bbabRural3m":"1.97","bbabUrban3m":"1.68","ebaCountry12m":null,"bbaCountry6m":"1.24","ebaUrban3m":"1.73","ebaCountry3m":"1.62","bbaUrban3m":"1.63","bbaCountry12m":null,"bbaRural6m":"1.54","bbabCountry6m":"1.29","bbaRural12m":null,"bbabRural6m":"1.69","ebaRural6m":"1.58","bbabUrban12m":null,"bbaCountry3m":"1.52","bbaUrban12m":null,"bbabUrban6m":"1.4"},{"bbabCountry12m":null,"date":"20250219","ebaCountry6m":"1.28","bbaUrban6m":"1.33","ebaRural12m":null,"bbaRural3m":"1.75","ebaRural3m":"1.9","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.5","ebaUrban6m":"1.39","bbabRural3m":"1.9","bbabUrban3m":"1.61","ebaCountry12m":null,"bbaCountry6m":"1.22","ebaUrban3m":"1.71","ebaCountry3m":"1.6","bbaUrban3m":"1.56","bbaCountry12m":null,"bbaRural6m":"1.52","bbabCountry6m":"1.27","bbaRural12m":null,"bbabRural6m":"1.67","ebaRural6m":"1.58","bbabUrban12m":null,"bbaCountry3m":"1.45","bbaUrban12m":null,"bbabUrban6m":"1.38"},{"bbabCountry12m":null,"date":"20250220","ebaCountry6m":"1.27","bbaUrban6m":"1.31","ebaRural12m":null,"bbaRural3m":"1.75","ebaRural3m":"1.9","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.49","ebaUrban6m":"1.38","bbabRural3m":"1.9","bbabUrban3m":"1.6","ebaCountry12m":null,"bbaCountry6m":"1.2","ebaUrban3m":"1.69","ebaCountry3m":"1.58","bbaUrban3m":"1.55","bbaCountry12m":null,"bbaRural6m":"1.5","bbabCountry6m":"1.25","bbaRural12m":null,"bbabRural6m":"1.65","ebaRural6m":"1.57","bbabUrban12m":null,"bbaCountry3m":"1.44","bbaUrban12m":null,"bbabUrban6m":"1.36"},{"bbabCountry12m":null,"date":"20250221","ebaCountry6m":"1.27","bbaUrban6m":"1.32","ebaRural12m":null,"bbaRural3m":"1.76","ebaRural3m":"1.9","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.5","ebaUrban6m":"1.38","bbabRural3m":"1.91","bbabUrban3m":"1.61","ebaCountry12m":null,"bbaCountry6m":"1.21","ebaUrban3m":"1.69","ebaCountry3m":"1.58","bbaUrban3m":"1.56","bbaCountry12m":null,"bbaRural6m":"1.51","bbabCountry6m":"1.26","bbaRural12m":null,"bbabRural6m":"1.66","ebaRural6m":"1.57","bbabUrban12m":null,"bbaCountry3m":"1.45","bbaUrban12m":null,"bbabUrban6m":"1.37"},{"bbabCountry12m":null,"date":"20250224","ebaCountry6m":"1.27","bbaUrban6m":"1.33","ebaRural12m":null,"bbaRural3m":"1.76","ebaRural3m":"1.9","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.5","ebaUrban6m":"1.38","bbabRural3m":"1.91","bbabUrban3m":"1.61","ebaCountry12m":null,"bbaCountry6m":"1.22","ebaUrban3m":"1.69","ebaCountry3m":"1.58","bbaUrban3m":"1.56","bbaCountry12m":null,"bbaRural6m":"1.52","bbabCountry6m":"1.27","bbaRural12m":null,"bbabRural6m":"1.67","ebaRural6m":"1.57","bbabUrban12m":null,"bbaCountry3m":"1.45","bbaUrban12m":null,"bbabUrban6m":"1.38"},{"bbabCountry12m":null,"date":"20250225","ebaCountry6m":"1.22","bbaUrban6m":"1.28","ebaRural12m":null,"bbaRural3m":"1.7","ebaRural3m":"1.85","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.45","ebaUrban6m":"1.33","bbabRural3m":"1.85","bbabUrban3m":"1.56","ebaCountry12m":null,"bbaCountry6m":"1.17","ebaUrban3m":"1.65","ebaCountry3m":"1.54","bbaUrban3m":"1.51","bbaCountry12m":null,"bbaRural6m":"1.47","bbabCountry6m":"1.22","bbaRural12m":null,"bbabRural6m":"1.62","ebaRural6m":"1.52","bbabUrban12m":null,"bbaCountry3m":"1.4","bbaUrban12m":null,"bbabUrban6m":"1.33"},{"bbabCountry12m":null,"date":"20250226","ebaCountry6m":"1.2","bbaUrban6m":"1.25","ebaRural12m":null,"bbaRural3m":"1.55","ebaRural3m":"1.75","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.28","ebaUrban6m":"1.31","bbabRural3m":"1.7","bbabUrban3m":"1.39","ebaCountry12m":null,"bbaCountry6m":"1.14","ebaUrban3m":"1.56","ebaCountry3m":"1.45","bbaUrban3m":"1.34","bbaCountry12m":null,"bbaRural6m":"1.44","bbabCountry6m":"1.19","bbaRural12m":null,"bbabRural6m":"1.59","ebaRural6m":"1.5","bbabUrban12m":null,"bbaCountry3m":"1.23","bbaUrban12m":null,"bbabUrban6m":"1.3"},{"bbabCountry12m":null,"date":"20250227","ebaCountry6m":"1.2","bbaUrban6m":"1.19","ebaRural12m":null,"bbaRural3m":"1.3","ebaRural3m":"1.7","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.05","ebaUrban6m":"1.31","bbabRural3m":"1.45","bbabUrban3m":"1.16","ebaCountry12m":null,"bbaCountry6m":"1.08","ebaUrban3m":"1.51","ebaCountry3m":"1.4","bbaUrban3m":"1.11","bbaCountry12m":null,"bbaRural6m":"1.38","bbabCountry6m":"1.13","bbaRural12m":null,"bbabRural6m":"1.53","ebaRural6m":"1.5","bbabUrban12m":null,"bbaCountry3m":"1.0","bbaUrban12m":null,"bbabUrban6m":"1.24"},{"bbabCountry12m":null,"date":"20250228","ebaCountry6m":"1.15","bbaUrban6m":"1.16","ebaRural12m":null,"bbaRural3m":"1.2","ebaRural3m":"1.7","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"0.95","ebaUrban6m":"1.26","bbabRural3m":"1.35","bbabUrban3m":"1.06","ebaCountry12m":null,"bbaCountry6m":"1.05","ebaUrban3m":"1.51","ebaCountry3m":"1.4","bbaUrban3m":"1.01","bbaCountry12m":null,"bbaRural6m":"1.35","bbabCountry6m":"1.1","bbaRural12m":null,"bbabRural6m":"1.5","ebaRural6m":"1.45","bbabUrban12m":null,"bbaCountry3m":"0.9","bbaUrban12m":null,"bbabUrban6m":"1.21"},{"bbabCountry12m":null,"date":"20250303","ebaCountry6m":"1.32","bbaUrban6m":"1.35","ebaRural12m":null,"bbaRural3m":"1.7","ebaRural3m":"1.95","ebaUrban12m":null,"bbabRural12m":null,"bbabCountry3m":"1.45","ebaUrban6m":"1.43","bbabRural3m":"1.85","bbabUrban3m":"1.56","ebaCountry12m":null,"bbaCountry6m":"1.24","ebaUrban3m":"1.76","ebaCountry3m":"1.65","bbaUrban3m":"1.51","bbaCountry12m":null,"bbaRural6m":"1.54","bbabCountry6m":"1.29","bbaRural12m":null,"bbabRural6m":"1.69","ebaRural6m":"1.65","bbabUrban12m":null,"bbaCountry3m":"1.4","bbaUrban12m":null,"bbabUrban6m":"1.4"}],"sign":"526c097b96b578e61274ee5f00d11219","message":"成功","sign_type":"1"}`

	// 6. 解析响应
	var baseResp BaseResponse
	if err := json.NewDecoder(bytes.NewReader(body)).Decode(&baseResp); err != nil {
		return nil, fmt.Errorf("decode response failed: %v", err)
	}
	if baseResp.Code != "10000" {
		err = fmt.Errorf("接口返回失败，code: %s, err: %s",baseResp.Code, baseResp.Message)
		utils.FileLog.Info(string(body))
		return nil, err
	}
	// var baseResp BaseResponse
	// if err := json.Unmarshal([]byte(bodystr), &baseResp); err != nil {
	// 	return nil, fmt.Errorf("decode response failed: %v", err)
	// }

	// 7. 验证响应签名（如果需要）
	// TODO: 根据实际需求实现响应签名验证
	// 解析响应
	//fmt.Println("响应",baseResp)
	return &baseResp, nil
}